{
  This file was automatically generated on 2013-11-27.

  Bindings Version 2.0.14

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletLEDStrip;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_LED_STRIP_DEVICE_IDENTIFIER = 231;

  BRICKLET_LED_STRIP_FUNCTION_SET_RGB_VALUES = 1;
  BRICKLET_LED_STRIP_FUNCTION_GET_RGB_VALUES = 2;
  BRICKLET_LED_STRIP_FUNCTION_SET_FRAME_DURATION = 3;
  BRICKLET_LED_STRIP_FUNCTION_GET_FRAME_DURATION = 4;
  BRICKLET_LED_STRIP_FUNCTION_GET_SUPPLY_VOLTAGE = 5;
  BRICKLET_LED_STRIP_FUNCTION_GET_IDENTITY = 255;

  BRICKLET_LED_STRIP_CALLBACK_FRAME_RENDERED = 6;


type
  TArray0To2OfUInt8 = array [0..2] of byte;
  TArray0To15OfUInt8 = array [0..15] of byte;

  TBrickletLEDStrip = class;
  TBrickletLEDStripNotifyFrameRendered = procedure(sender: TBrickletLEDStrip; const length2: word) of object;

  /// <summary>
  ///  Device to control up to 320 RGB LEDs
  /// </summary>
  TBrickletLEDStrip = class(TDevice)
  private
    frameRenderedCallback: TBrickletLEDStripNotifyFrameRendered;
  protected
    procedure CallbackWrapperFrameRendered(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid__: string; ipcon_: TIPConnection);

    /// <summary>
    ///  Sets the *rgb* values for the LEDs with the given *length* starting 
    ///  from *index*.
    ///  
    ///  The maximum length is 16, the index goes from 0 to 319 and the rgb values
    ///  have 8 bits each.
    ///  
    ///  Example: If you set index to 5, length to 3, r to [255, 0, 0], 
    ///  g to [0, 255, 0] and b to [0, 0, 255] the LEDs with index 5, 6
    ///  and 7 will get the color red, green and blue respectively.
    ///  
    ///  The colors will be transfered to actual LEDs when the next
    ///  frame duration ends, see <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetFrameDuration"/>.
    ///  
    ///  Generic approach: 
    ///  
    ///  * Set the frame duration to a value that represents
    ///    the number of frames per second you want to achieve. 
    ///  * Set all of the LED colors for one frame.
    ///  * Wait for the <see cref="BrickletLEDStrip.TBrickletLEDStrip.FrameRendered"/> callback.
    ///  * Set all of the LED colors for next frame.
    ///  * Wait for the <see cref="BrickletLEDStrip.TBrickletLEDStrip.FrameRendered"/> callback.
    ///  * and so on.
    ///  
    ///  This approach ensures that you can change the LED colors with
    ///  a fixed frame rate.
    ///  
    ///  The actual number of controllable LEDs depends on the number of free
    ///  Bricklet ports. See :ref:`here &lt;led_strip_bricklet_ram_constraints&gt;` for more
    ///  information. A call of <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetRGBValues"/> with index + length above the
    ///  bounds is ignored completely.
    /// </summary>
    procedure SetRGBValues(const index: word; const length2: byte; const r: TArray0To15OfUInt8; const g: TArray0To15OfUInt8; const b: TArray0To15OfUInt8); virtual;

    /// <summary>
    ///  Returns the rgb with the given *length* starting from the
    ///  given *index*.
    ///  
    ///  The values are the last values that were set by <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetRGBValues"/>.
    /// </summary>
    procedure GetRGBValues(const index: word; const length2: byte; out r: TArray0To15OfUInt8; out g: TArray0To15OfUInt8; out b: TArray0To15OfUInt8); virtual;

    /// <summary>
    ///  Sets the frame duration in ms.
    ///  
    ///  Example: If you want to achieve 20 frames per second, you should
    ///  set the frame duration to 50ms (50ms * 20 = 1 second). 
    ///  
    ///  For an explanation of the general approach see <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetRGBValues"/>.
    ///  
    ///  Default value: 100ms (10 frames per second).
    /// </summary>
    procedure SetFrameDuration(const duration: word); virtual;

    /// <summary>
    ///  Returns the frame duration as set by <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetFrameDuration"/>.
    /// </summary>
    function GetFrameDuration: word; virtual;

    /// <summary>
    ///  Returns the current supply voltage of the LEDs. The voltage is given in mV.
    /// </summary>
    function GetSupplyVoltage: word; virtual;

    /// <summary>
    ///  Returns the UID, the UID where the Bricklet is connected to, 
    ///  the position, the hardware and firmware version as well as the
    ///  device identifier.
    ///  
    ///  The position can be 'a', 'b', 'c' or 'd'.
    ///  
    ///  The device identifiers can be found :ref:`here &lt;device_identifier&gt;`.
    ///  
    ///  .. versionadded:: 2.0.0~(Plugin)
    /// </summary>
    procedure GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word); override;

    /// <summary>
    ///  This callback is triggered directly after a new frame is rendered.
    ///  
    ///  You should send the data for the next frame directly after this callback
    ///  was triggered.
    ///  
    ///  For an explanation of the general approach see <see cref="BrickletLEDStrip.TBrickletLEDStrip.SetRGBValues"/>.
    /// </summary>
    property OnFrameRendered: TBrickletLEDStripNotifyFrameRendered read frameRenderedCallback write frameRenderedCallback;
  end;

implementation

constructor TBrickletLEDStrip.Create(const uid__: string; ipcon_: TIPConnection);
begin
  inherited Create(uid__, ipcon_);
  apiVersion[0] := 2;
  apiVersion[1] := 0;
  apiVersion[2] := 0;

  responseExpected[BRICKLET_LED_STRIP_FUNCTION_SET_RGB_VALUES] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_LED_STRIP_FUNCTION_GET_RGB_VALUES] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_LED_STRIP_FUNCTION_SET_FRAME_DURATION] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_LED_STRIP_FUNCTION_GET_FRAME_DURATION] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_LED_STRIP_FUNCTION_GET_SUPPLY_VOLTAGE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_LED_STRIP_CALLBACK_FRAME_RENDERED] := DEVICE_RESPONSE_EXPECTED_ALWAYS_FALSE;
  responseExpected[BRICKLET_LED_STRIP_FUNCTION_GET_IDENTITY] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;

  callbackWrappers[BRICKLET_LED_STRIP_CALLBACK_FRAME_RENDERED] := {$ifdef FPC}@{$endif}CallbackWrapperFrameRendered;
end;

procedure TBrickletLEDStrip.SetRGBValues(const index: word; const length2: byte; const r: TArray0To15OfUInt8; const g: TArray0To15OfUInt8; const b: TArray0To15OfUInt8);
var request: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_SET_RGB_VALUES, 59);
  LEConvertUInt16To(index, 8, request);
  LEConvertUInt8To(length2, 10, request);
  for i := 0 to Length(r) - 1 do LEConvertUInt8To(r[i], 11 + (i * 1), request);
  for i := 0 to Length(g) - 1 do LEConvertUInt8To(g[i], 27 + (i * 1), request);
  for i := 0 to Length(b) - 1 do LEConvertUInt8To(b[i], 43 + (i * 1), request);
  SendRequest(request);
end;

procedure TBrickletLEDStrip.GetRGBValues(const index: word; const length2: byte; out r: TArray0To15OfUInt8; out g: TArray0To15OfUInt8; out b: TArray0To15OfUInt8);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_GET_RGB_VALUES, 11);
  LEConvertUInt16To(index, 8, request);
  LEConvertUInt8To(length2, 10, request);
  response := SendRequest(request);
  for i := 0 to 15 do r[i] := LEConvertUInt8From(8 + (i * 1), response);
  for i := 0 to 15 do g[i] := LEConvertUInt8From(24 + (i * 1), response);
  for i := 0 to 15 do b[i] := LEConvertUInt8From(40 + (i * 1), response);
end;

procedure TBrickletLEDStrip.SetFrameDuration(const duration: word);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_SET_FRAME_DURATION, 10);
  LEConvertUInt16To(duration, 8, request);
  SendRequest(request);
end;

function TBrickletLEDStrip.GetFrameDuration: word;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_GET_FRAME_DURATION, 8);
  response := SendRequest(request);
  result := LEConvertUInt16From(8, response);
end;

function TBrickletLEDStrip.GetSupplyVoltage: word;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_GET_SUPPLY_VOLTAGE, 8);
  response := SendRequest(request);
  result := LEConvertUInt16From(8, response);
end;

procedure TBrickletLEDStrip.GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_LED_STRIP_FUNCTION_GET_IDENTITY, 8);
  response := SendRequest(request);
  uid := LEConvertStringFrom(8, 8, response);
  connectedUid := LEConvertStringFrom(16, 8, response);
  position := LEConvertCharFrom(24, response);
  for i := 0 to 2 do hardwareVersion[i] := LEConvertUInt8From(25 + (i * 1), response);
  for i := 0 to 2 do firmwareVersion[i] := LEConvertUInt8From(28 + (i * 1), response);
  deviceIdentifier := LEConvertUInt16From(31, response);
end;

procedure TBrickletLEDStrip.CallbackWrapperFrameRendered(const packet: TByteArray);
var length2: word;
begin
  if (Assigned(frameRenderedCallback)) then begin
    length2 := LEConvertUInt16From(8, packet);
    frameRenderedCallback(self, length2);
  end;
end;

end.
