{
  This file was automatically generated on 2013-11-27.

  Bindings Version 2.0.14

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletRemoteSwitch;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_REMOTE_SWITCH_DEVICE_IDENTIFIER = 235;

  BRICKLET_REMOTE_SWITCH_FUNCTION_SWITCH_SOCKET = 1;
  BRICKLET_REMOTE_SWITCH_FUNCTION_GET_SWITCHING_STATE = 2;
  BRICKLET_REMOTE_SWITCH_FUNCTION_SET_REPEATS = 4;
  BRICKLET_REMOTE_SWITCH_FUNCTION_GET_REPEATS = 5;
  BRICKLET_REMOTE_SWITCH_FUNCTION_GET_IDENTITY = 255;

  BRICKLET_REMOTE_SWITCH_CALLBACK_SWITCHING_DONE = 3;

  BRICKLET_REMOTE_SWITCH_SWITCH_TO_OFF = 0;
  BRICKLET_REMOTE_SWITCH_SWITCH_TO_ON = 1;
  BRICKLET_REMOTE_SWITCH_SWITCHING_STATE_READY = 0;
  BRICKLET_REMOTE_SWITCH_SWITCHING_STATE_BUSY = 1;

type
  TArray0To2OfUInt8 = array [0..2] of byte;

  TBrickletRemoteSwitch = class;
  TBrickletRemoteSwitchNotifySwitchingDone = procedure(sender: TBrickletRemoteSwitch) of object;

  /// <summary>
  ///  Device that controls mains switches remotely
  /// </summary>
  TBrickletRemoteSwitch = class(TDevice)
  private
    switchingDoneCallback: TBrickletRemoteSwitchNotifySwitchingDone;
  protected
    procedure CallbackWrapperSwitchingDone(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid__: string; ipcon_: TIPConnection);

    /// <summary>
    ///  To switch a socket you have to give the house code, receiver code and the
    ///  state (on or off) you want to switch to.
    ///  
    ///  A detailed description on how you can find the house and receiver code
    ///  can be found :ref:`here &lt;remote_switch_bricklet_house_and_receiver_code&gt;`.
    /// </summary>
    procedure SwitchSocket(const houseCode: byte; const receiverCode: byte; const switchTo: byte); virtual;

    /// <summary>
    ///  Returns the current switching state. If the current state is busy, the
    ///  Bricklet is currently sending a code to switch a socket. It will not
    ///  accept any calls of <see cref="BrickletRemoteSwitch.TBrickletRemoteSwitch.SwitchSocket"/> until the state changes to ready.
    ///  
    ///  How long the switching takes is dependent on the number of repeats, see
    ///  <see cref="BrickletRemoteSwitch.TBrickletRemoteSwitch.SetRepeats"/>.
    /// </summary>
    function GetSwitchingState: byte; virtual;

    /// <summary>
    ///  Sets the number of times the code is send when <see cref="BrickletRemoteSwitch.TBrickletRemoteSwitch.SwitchSocket"/> is called.
    ///  The repeats basically correspond to the amount of time that a button of the
    ///  remote is pressed. 
    ///  
    ///  Some dimmers are controlled by the length of a button pressed,
    ///  this can be simulated by increasing the repeats.
    ///  
    ///  The default value is 5.
    /// </summary>
    procedure SetRepeats(const repeats: byte); virtual;

    /// <summary>
    ///  Returns the number of repeats as set by <see cref="BrickletRemoteSwitch.TBrickletRemoteSwitch.SetRepeats"/>.
    /// </summary>
    function GetRepeats: byte; virtual;

    /// <summary>
    ///  Returns the UID, the UID where the Bricklet is connected to, 
    ///  the position, the hardware and firmware version as well as the
    ///  device identifier.
    ///  
    ///  The position can be 'a', 'b', 'c' or 'd'.
    ///  
    ///  The device identifiers can be found :ref:`here &lt;device_identifier&gt;`.
    ///  
    ///  .. versionadded:: 2.0.0~(Plugin)
    /// </summary>
    procedure GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word); override;

    /// <summary>
    ///  This callback is called whenever the switching state changes
    ///  from busy to ready, see <see cref="BrickletRemoteSwitch.TBrickletRemoteSwitch.GetSwitchingState"/>.
    /// </summary>
    property OnSwitchingDone: TBrickletRemoteSwitchNotifySwitchingDone read switchingDoneCallback write switchingDoneCallback;
  end;

implementation

constructor TBrickletRemoteSwitch.Create(const uid__: string; ipcon_: TIPConnection);
begin
  inherited Create(uid__, ipcon_);
  apiVersion[0] := 2;
  apiVersion[1] := 0;
  apiVersion[2] := 0;

  responseExpected[BRICKLET_REMOTE_SWITCH_FUNCTION_SWITCH_SOCKET] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_REMOTE_SWITCH_FUNCTION_GET_SWITCHING_STATE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_REMOTE_SWITCH_CALLBACK_SWITCHING_DONE] := DEVICE_RESPONSE_EXPECTED_ALWAYS_FALSE;
  responseExpected[BRICKLET_REMOTE_SWITCH_FUNCTION_SET_REPEATS] := DEVICE_RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_REMOTE_SWITCH_FUNCTION_GET_REPEATS] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_REMOTE_SWITCH_FUNCTION_GET_IDENTITY] := DEVICE_RESPONSE_EXPECTED_ALWAYS_TRUE;

  callbackWrappers[BRICKLET_REMOTE_SWITCH_CALLBACK_SWITCHING_DONE] := {$ifdef FPC}@{$endif}CallbackWrapperSwitchingDone;
end;

procedure TBrickletRemoteSwitch.SwitchSocket(const houseCode: byte; const receiverCode: byte; const switchTo: byte);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REMOTE_SWITCH_FUNCTION_SWITCH_SOCKET, 11);
  LEConvertUInt8To(houseCode, 8, request);
  LEConvertUInt8To(receiverCode, 9, request);
  LEConvertUInt8To(switchTo, 10, request);
  SendRequest(request);
end;

function TBrickletRemoteSwitch.GetSwitchingState: byte;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REMOTE_SWITCH_FUNCTION_GET_SWITCHING_STATE, 8);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

procedure TBrickletRemoteSwitch.SetRepeats(const repeats: byte);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REMOTE_SWITCH_FUNCTION_SET_REPEATS, 9);
  LEConvertUInt8To(repeats, 8, request);
  SendRequest(request);
end;

function TBrickletRemoteSwitch.GetRepeats: byte;
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REMOTE_SWITCH_FUNCTION_GET_REPEATS, 8);
  response := SendRequest(request);
  result := LEConvertUInt8From(8, response);
end;

procedure TBrickletRemoteSwitch.GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TVersionNumber; out firmwareVersion: TVersionNumber; out deviceIdentifier: word);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreateRequestPacket(self, BRICKLET_REMOTE_SWITCH_FUNCTION_GET_IDENTITY, 8);
  response := SendRequest(request);
  uid := LEConvertStringFrom(8, 8, response);
  connectedUid := LEConvertStringFrom(16, 8, response);
  position := LEConvertCharFrom(24, response);
  for i := 0 to 2 do hardwareVersion[i] := LEConvertUInt8From(25 + (i * 1), response);
  for i := 0 to 2 do firmwareVersion[i] := LEConvertUInt8From(28 + (i * 1), response);
  deviceIdentifier := LEConvertUInt16From(31, response);
end;

procedure TBrickletRemoteSwitch.CallbackWrapperSwitchingDone(const packet: TByteArray);
begin
  Assert(packet <> nil); { Avoid 'Parameter not used' warning }
  if (Assigned(switchingDoneCallback)) then begin
    switchingDoneCallback(self);
  end;
end;

end.
